; ###################################################################################################
; #                                        JV-ODE Car Demo                                          #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE Car Demo - Detailed Collision Info"

Graphics3D 800,600,0,2

Global CMass#=200.0				; ### Car Mass
Global WMass#=14.0				; ### Wheel Mass

Global WorldERP#=1.0			; ### World Error Correction
Global WorldFriction#=70.0		; ### World Friction

Global Torque#=24.0				; ### Joint Torque
Global SuspensionHS#=0.01		; ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Global Force#=0.0
Global Steer#=0.0

Global Car
Global CGeom
Global CMesh

Global CarStartY=20

Global RampGeom
Global BlueGeom
Global PlaneGeom

Global GeomFlash
Global GeomStatus
Global CheckGeomID
Global CheckGeomTag$

Dim Wheel(4)
Dim WGeom(4)
Dim Joint(4)

Dim BoxGeom(20)
Dim SphereGeom(20)
Dim CapsuleGeom(20)

Dim Pointer(2)
Dim GeomTag$(2)

Dim GeomX#(2)
Dim GeomY#(2)
Dim GeomZ#(2)
Dim GeomGeom(2)
Dim GeomCCount(2)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

; ### Create Car Body

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
Car=ode\body
dBodySetPosition(ode\body,0,CarStartY,0)
dBodySetRotation(ode\body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,CMass,3,1,4)
dBodySetMass(ode\body,mass)
dMassDestroy(mass)
ode\geom=dCreateBox(Space,3,1,4)
CGeom=ode\geom
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
CMesh=ode\mesh
ScaleMesh ode\mesh,1.5,0.5,2
EntityColor ode\mesh,40,80,255

; ### Create Wheels

For count=1 To 4
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	Wheel(count)=ode\body
	dBodySetPosition(ode\body,0,0,0)
	dBodySetRotation(ode\body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	ode\geom=dCreateSphere(Space,0.7)
	WGeom(count)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,0.7,0.25,0.7
	EntityColor ode\mesh,255,255,255
Next

dBodySetPosition(Wheel(1),-2,CarStartY-0.5,+2)
dBodySetPosition(Wheel(2),+2,CarStartY-0.5,+2)
dBodySetPosition(Wheel(3),-2,CarStartY-0.5,-2)
dBodySetPosition(Wheel(4),+2,CarStartY-0.5,-2)

; ### Create Objects

SeedRnd MilliSecs()

For cubes=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),20,Rand(-100,100))
	dBodySetRotation(ode\body,0,0,0)
	ode\geom=dCreateBox(Space,1,1,1)
	BoxGeom(cubes)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCube()
	ScaleMesh ode\mesh,0.5,0.5,0.5
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

For spheres=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode\body,0,0,0)
	ode\geom=dCreateSphere(Space,4)
	SphereGeom(spheres)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateSphere()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

For capsules=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),40,Rand(-100,100))
	dBodySetRotation(ode\body,90,0,0)
	ode\geom=dCreateCapsule(Space,1,8)
	CapsuleGeom(capsules)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,1,5,1
	RotateMesh ode\mesh,90,0,0	; ### Align Blitz3D Cylinder To ODE Capsule
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global CameraPivot=CreatePivot()

Global Camera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

; ### Create Plane

PlaneGeom=dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### ODE Static Object (Blue)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,2,8,40)
BlueGeom=ode\geom
dGeomSetPosition(ode\geom,20,5,40)
dGeomSetRotation(ode\geom,-10,8,4)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,1,4,20
EntityColor ode\mesh,0,100,200

; ### ODE Static Object (Red)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,18,0.2,40)
RampGeom=ode\geom
dGeomSetPosition(ode\geom,-20,4.9,40)
dGeomSetRotation(ode\geom,-14,0,0)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,9,0.1,20
EntityColor ode\mesh,200,0,100

; ### Blitz3D Collision Pointers

For count=1 To 2
	Pointer(count)=CreateSphere()
	ScaleMesh Pointer(count),0.5,0.5,0.5
	EntityColor Pointer(count),255,255,100
	EntityAlpha Pointer(count),0.7
Next

; ###################################################################################################

SetupCar()

CheckGeomID=CGeom
CheckGeomTag="Car Body"

While Not KeyHit(1)

	UpdateKeys()

	UpdateCar()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	GeomCollisionInfo(CheckGeomID)

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 0,50,"Force:"+Force
	Text 0,65,"Torque:"+Torque
	Text 240,0,"F1 - Change Detected Geom: "+CheckGeomTag
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Car"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function SetupCar()

For count=1 To 4
	Joint(count)=dJointCreateHinge2(World,0)
	dJointAttach(Joint(count),Car,Wheel(count))
	dJointSetHinge2Anchor(Joint(count),dBodyGetPositionX(Wheel(count)),dBodyGetPositionY(Wheel(count)),dBodyGetPositionZ(Wheel(count)))
	dJointSetHinge2Axis1(Joint(count),0,1,0)
	dJointSetHinge2Axis2(Joint(count),-1,0,0)
	dJointSetHinge2Param(Joint(count),dParamSuspensionERP,0.8)
	dJointSetHinge2Param(Joint(count),dParamSuspensionCFM,SuspensionHS)
	If count>2
		dJointSetHinge2Param(Joint(count),dParamLoStop,0)
		dJointSetHinge2Param(Joint(count),dParamHiStop,0)
		End If
Next

End Function

; ###################################################################################################

Function GeomCollisionInfo(GeomID)

If dGeom1CountCollisions(GeomID)>0
	GeomX(1)=dGeom1CollisionX(GeomID,1)
	GeomY(1)=dGeom1CollisionY(GeomID,1)
	GeomZ(1)=dGeom1CollisionZ(GeomID,1)
	GeomGeom(1)=dGeom1CollisionGeom(GeomID,1)
	GeomCCount(1)=dGeom1CountCollisions(GeomID)
	PositionEntity Pointer(1),GeomX(1),GeomY(1),GeomZ(1)
	End If

If dGeom2CountCollisions(GeomID)>0
	GeomX(2)=dGeom2CollisionX(GeomID,1)
	GeomY(2)=dGeom2CollisionY(GeomID,1)
	GeomZ(2)=dGeom2CollisionZ(GeomID,1)
	GeomGeom(2)=dGeom2CollisionGeom(GeomID,1)
	GeomCCount(2)=dGeom2CountCollisions(GeomID)
	PositionEntity Pointer(2),GeomX(2),GeomY(2),GeomZ(2)
	End If

For count=1 To 2
	GeomTag(count)="None"
	If GeomGeom(count)=CGeom Then GeomTag(count)="Car Body"
	If GeomGeom(count)=WGeom(1) Then GeomTag(count)="Wheel 1"
	If GeomGeom(count)=WGeom(2) Then GeomTag(count)="Wheel 2"
	If GeomGeom(count)=WGeom(3) Then GeomTag(count)="Wheel 3"
	If GeomGeom(count)=WGeom(4) Then GeomTag(count)="Wheel 4"
	If GeomGeom(count)=BlueGeom Then GeomTag(count)="Blue Block"
	If GeomGeom(count)=RampGeom Then GeomTag(count)="Red Ramp"
	If GeomGeom(count)=PlaneGeom Then GeomTag(count)="Ground Plane"
	For count2=1 To 20
		If GeomGeom(count)=BoxGeom(count2) Then GeomTag(count)="Box "+count2
		If GeomGeom(count)=SphereGeom(count2) Then GeomTag(count)="Sphere "+count2
		If GeomGeom(count)=CapsuleGeom(count2) Then GeomTag(count)="Capsule "+count2
	Next
Next

Text 0,100,"Geom 1 Data (Geom Size > "+CheckGeomTag+")"
Text 0,125,"Collision X: "+GeomX(1)
Text 0,140,"Collision Y: "+GeomY(1)
Text 0,155,"Collision Z: "+GeomZ(1)
Text 0,170,"Collision Geom: "+GeomTag(1)
Text 0,185,"Collision Count: "+GeomCCount(1)

Text 0,220,"Geom 2 Data (Geom Size < "+CheckGeomTag+")"
Text 0,245,"Collision X: "+GeomX(2)
Text 0,260,"Collision Y: "+GeomY(2)
Text 0,275,"Collision Z: "+GeomZ(2)
Text 0,290,"Collision Geom: "+GeomTag(2)
Text 0,305,"Collision Count: "+GeomCCount(2)

GeomFlash=GeomFlash+1
If GeomFlash>100 Then GeomFlash=0

For count=1 To 2
	ScaleEntity Pointer(count),0.5+(GeomFlash*0.01),0.5+(GeomFlash*0.01),0.5+(GeomFlash*0.01)
	EntityColor Pointer(count),255,255-(GeomFlash*2),100
Next

End Function

; ###################################################################################################

Function UpdateKeys()

If KeyDown(30)=1
	Force=Force+0.08
	If Force>30 Then Force=30
	End If

If KeyDown(44)=1
	Force=Force-0.1
	If Force<-10 Then Force=-10
	If Force>0 Then Force=Force*0.97
	End If

If KeyDown(30)=0 And KeyDown(44)=0 Then Force=Force*0.99

If KeyDown(203)=1 Or KeyDown(205)=1
	If KeyDown(203)=1
		Steer=0.5
		Else
		Steer=-0.5
		End If
	Else
	Steer=0.0
	End If

If KeyHit(57)=1
	Force=0.0
	dBodySetRotation(Car,dGeomGetPitch(CGeom),dGeomGetYaw(CGeom),0)
	End If

If KeyHit(59)
	GeomStatus=GeomStatus+1
	If GeomStatus>7 Then GeomStatus=0
	If GeomStatus=0 Then CheckGeomID=CGeom : CheckGeomTag="Car Body"
	If GeomStatus=1 Then CheckGeomID=WGeom(1) : CheckGeomTag="Wheel 1 (F/L)"
	If GeomStatus=2 Then CheckGeomID=WGeom(2) : CheckGeomTag="Wheel 2 (F/R)"
	If GeomStatus=3 Then CheckGeomID=WGeom(3) : CheckGeomTag="Wheel 3 (R/L)"
	If GeomStatus=4 Then CheckGeomID=WGeom(4) : CheckGeomTag="Wheel 4 (R/R)"
	If GeomStatus=5 Then CheckGeomID=BlueGeom : CheckGeomTag="Blue Block"
	If GeomStatus=6 Then CheckGeomID=RampGeom : CheckGeomTag="Red Ramp"
	If GeomStatus=7 Then CheckGeomID=PlaneGeom : CheckGeomTag="Ground Plane"
	For count=1 To 2
		GeomTag(count)="None"
		GeomX(count)=0.0
		GeomY(count)=0.0
		GeomZ(count)=0.0
		GeomGeom(count)=0
		GeomCCount(count)=0
	Next
	End If

End Function

; ###################################################################################################

Function UpdateCar()

For count=1 To 4
	dBodyEnable(Wheel(count))
Next

dBodyEnable(Car)

For count=1 To 4
	dJointSetHinge2Param(Joint(count),dParamVel2,Force)
	dJointSetHinge2Param(Joint(count),dParamFMax2,Torque)
Next

For count=1 To 2
	angle#=Steer-dJointGetHinge2Angle1(Joint(count))
	dJointSetHinge2Param(Joint(count),dParamVel,angle)
	dJointSetHinge2Param(Joint(count),dParamFMax,400)
Next

End Function

; ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,3,-12

PivotX#=Cos(EntityYaw(CMesh)-90)+EntityX(CMesh)
PivotY#=EntityY(CMesh)
PivotZ#=Sin(EntityYaw(CMesh)-90)+EntityZ(CMesh)

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
