; ###################################################################################################
; #                                        JV-ODE Car Demo                                          #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE Car Demo - Duo"

Graphics3D 800,600,0,2

Global CMass#=200.0				; ### Car Mass
Global WMass#=14.0				; ### Wheel Mass

Global WorldERP#=1.0			; ### World Error Correction
Global WorldFriction#=70.0		; ### World Friction

Global Torque#=24.0				; ### Joint Torque
Global SuspensionHS#=0.01		; ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Dim Force#(2)
Dim Steer#(2)

Dim Car(2)
Dim CGeom(2)
Dim CMesh(2)

Global CarStartY=20

Global ControlTimer=0
Global SteeringTimer=0

Dim Wheel(2,4)
Dim WGeom(2,4)
Dim Joint(2,4)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

For carID=1 To 2
	; ### Create Car Body

	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	Car(carID)=ode\body
	dBodySetPosition(ode\body,-10+(carID*10),CarStartY,0)
	dBodySetRotation(ode\body,0,0,0)
	mass=dMassCreate()
	dMassSetBoxTotal(mass,CMass,3,1,4)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	ode\geom=dCreateBox(Space,3,1,4)
	CGeom(carID)=ode\geom
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCube()
	CMesh(carID)=ode\mesh
	ScaleMesh ode\mesh,1.5,0.5,2
	If carID=1
		EntityColor ode\mesh,40,80,255
		Else
		EntityColor ode\mesh,40,255,80
		End If

	; ### Create Wheels

	For count=1 To 4
		ode.ODEGeom=New ODEGeom
		ode\body=dBodyCreate(World)
		Wheel(carID,count)=ode\body
		dBodySetPosition(ode\body,0,0,0)
		dBodySetRotation(ode\body,0,0,90)
		mass=dMassCreate()
		dMassSetSphereTotal(mass,WMass,0.7)
		dBodySetMass(ode\body,mass)
		dMassDestroy(mass)
		ode\geom=dCreateSphere(Space,0.7)
		WGeom(carID,count)=ode\geom
		dGeomSetBody(ode\geom,ode\body)
		ode\mesh=CreateCylinder()
		ScaleMesh ode\mesh,0.7,0.25,0.7
		EntityColor ode\mesh,255,255,255
	Next

	dBodySetPosition(Wheel(carID,1),-12+(carID*10),CarStartY-0.5,+2)
	dBodySetPosition(Wheel(carID,2),-8+(carID*10),CarStartY-0.5,+2)
	dBodySetPosition(Wheel(carID,3),-12+(carID*10),CarStartY-0.5,-2)
	dBodySetPosition(Wheel(carID,4),-8+(carID*10),CarStartY-0.5,-2)
Next

; ### Create Objects

SeedRnd MilliSecs()

For spheres=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode\body,0,0,0)
	ode\geom=dCreateSphere(Space,4)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateSphere()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

For capsules=1 To 20
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode\body,90,0,0)
	ode\geom=dCreateCapsule(Space,1,8)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCylinder()
	ScaleMesh ode\mesh,1,5,1
	RotateMesh ode\mesh,90,0,0	; ### Align Blitz3D Cylinder To ODE Capsule
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global CameraPivot=CreatePivot()

Global Camera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### Blitz3D Static Object (White)

Global Blitz3DObject=CreateCube()
ScaleMesh Blitz3DObject,1,4,20
PositionEntity Blitz3DObject,10,5,40
RotateEntity Blitz3DObject,-10,8,4
EntityColor Blitz3DObject,255,255,255

; ### ODE Static Object (Blue)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,2,8,40)
dGeomSetPosition(ode\geom,20,5,40)
dGeomSetRotation(ode\geom,-10,8,4)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,1,4,20
EntityColor ode\mesh,0,100,200

; ### ODE Static Object (Red)

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,18,0.2,40)
dGeomSetPosition(ode\geom,-20,4.9,40)
dGeomSetRotation(ode\geom,-14,0,0)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,9,0.1,20
EntityColor ode\mesh,200,0,100

; ###################################################################################################

SetupCars()

While Not KeyHit(1)

	UpdateKeys()

	UpdateCars()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 0,50,"Force Car 1:"+Force(1)
	Text 0,65,"Force Car 2:"+Force(2)
	Text 0,80,"Torque:"+Torque
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Car"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function SetupCars()

For carID=1 To 2
	For count=1 To 4
		Joint(carID,count)=dJointCreateHinge2(World,0)
		dJointAttach(Joint(carID,count),Car(carID),Wheel(carID,count))
		dJointSetHinge2Anchor(Joint(carID,count),dBodyGetPositionX(Wheel(carID,count)),dBodyGetPositionY(Wheel(carID,count)),dBodyGetPositionZ(Wheel(carID,count)))
		dJointSetHinge2Axis1(Joint(carID,count),0,1,0)
		dJointSetHinge2Axis2(Joint(carID,count),-1,0,0)
		dJointSetHinge2Param(Joint(carID,count),dParamSuspensionERP,0.8)
		dJointSetHinge2Param(Joint(carID,count),dParamSuspensionCFM,SuspensionHS)
		If count>2
			dJointSetHinge2Param(Joint(carID,count),dParamLoStop,0)
			dJointSetHinge2Param(Joint(carID,count),dParamHiStop,0)
			End If
	Next
Next

End Function

; ###################################################################################################

Function UpdateKeys()

; ### Player Control - Car 1

If KeyDown(30)=1
	Force(1)=Force(1)+0.08
	If Force(1)>30 Then Force(1)=30
	End If

If KeyDown(44)=1
	Force(1)=Force(1)-0.1
	If Force(1)<-10 Then Force(1)=-10
	If Force(1)>0 Then Force(1)=Force(1)*0.97
	End If

If KeyDown(30)=0 And KeyDown(44)=0 Then Force(1)=Force(1)*0.99

If KeyDown(203)=1 Or KeyDown(205)=1
	If KeyDown(203)=1
		Steer(1)=0.5
		Else
		Steer(1)=-0.5
		End If
	Else
	Steer(1)=0.0
	End If

If KeyHit(57)=1
	Force(1)=0.0
	dBodySetRotation(Car(1),dGeomGetPitch(CGeom(1)),dGeomGetYaw(CGeom(1)),0)
	End If

; ### Simple AI - Car 2

If ControlTimer>100
	SeedRnd MilliSecs()
	AIKey=Rand(0,100)

	If AIKey<40
		Force(2)=Force(2)+0.8
		If Force(2)>10 Then Force(2)=10
		End If

	If AIKey=>40 Then Force(2)=Force(2)*0.99

	ControlTimer=0
	End If

If SteeringTimer>100
	SeedRnd MilliSecs()
	AIKey=Rand(0,100)

	Steer(2)=0.0
	If AIKey<30 Then Steer(2)=0.5
	If AIKey>70 Then Steer(2)=-0.5

	SteeringTimer=0
	End If

ControlTimer=ControlTimer+1
SteeringTimer=SteeringTimer+1

End Function

; ###################################################################################################

Function UpdateCars()

For carID=1 To 2
	For count=1 To 4
		dBodyEnable(Wheel(carID,count))
	Next

	dBodyEnable(Car(carID))

	For count=1 To 4
		dJointSetHinge2Param(Joint(carID,count),dParamVel2,Force(carID))
		dJointSetHinge2Param(Joint(carID,count),dParamFMax2,Torque)
	Next

	For count=1 To 2
		angle#=Steer(carID)-dJointGetHinge2Angle1(Joint(carID,count))
		dJointSetHinge2Param(Joint(carID,count),dParamVel,angle)
		dJointSetHinge2Param(Joint(carID,count),dParamFMax,400)
	Next
Next

End Function

; ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,3,-12

PivotX#=Cos(EntityYaw(CMesh(1))-90)+EntityX(CMesh(1))
PivotY#=EntityY(CMesh(1))
PivotZ#=Sin(EntityYaw(CMesh(1))-90)+EntityZ(CMesh(1))

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh(1)

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
