; ###################################################################################################
; #                                  JV-ODE - Geom Friction Demo                                    #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - Geom Friction Demo"

Graphics3D 800,600,0,2

Dim Cube(10)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.0)
dContactSetMu(40)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,45,-30
RotateEntity Camera,30,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.5

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

AddObjects()

While Not KeyHit(1)

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 10,170,"Geom Friction:"

	For count=1 To 10
		Text 158+(count*42),170,dGeomContactGetMu(Cube(count))
	Next

	Text 10,450,"Geom Friction (Ramp): "+dGeomContactGetMu(Cube(0))
	Text 10,510,"World Friction (Plane): "+dContactGetMu()

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function AddObjects()

ode.ODEGeom=New ODEGeom
ode\geom=dCreateBox(Space,100,1,80)
Cube(0)=ode\geom
dGeomSetPosition(ode\geom,0,8,40)
dGeomSetRotation(ode\geom,-8,0,0)
dGeomContactSetMu(ode\geom,20.0)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,50,0.5,40
EntityColor ode\mesh,0,100,200
EntityShininess ode\mesh,0.7

For count=1 To 10
	ode.ODEGeom=New ODEGeom
	ode\body=dBodyCreate(World)
	dBodySetPosition(ode\body,-55+(10*count),30,75)
	dBodySetRotation(ode\body,0,0,0)
	mass=dMassCreate()
	dMassSetBoxTotal(mass,300,8,8,8)
	dBodySetMass(ode\body,mass)
	dMassDestroy(mass)
	dBodySetAutoDisableFlag(ode\body,1)
	ode\geom=dCreateBox(Space,8,8,8)
	Cube(count)=ode\geom
	dGeomContactSetMu(ode\geom,count*0.5)
	dGeomSetBody(ode\geom,ode\body)
	ode\mesh=CreateCube()
	ScaleMesh ode\mesh,4,4,4
	EntityColor ode\mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode\mesh,0.7
Next

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
