; ###################################################################################################
; #                                     JV-ODE - RagDoll Demo                                       #
; #                                  Code by Jim Williams (VIP3R)                                   #
; #                              Devious Codeworks - Copyright  2008                               #
; ###################################################################################################

Include "JV-ODE.bb"

AppTitle "JV-ODE - RagDoll Demo"

Graphics3D 800,600,0,2

Global RagDollMax=3
Global ODEJointMax=14

Dim RDBody(RagDollMax,ODEJointMax)
Dim RDJoint(RagDollMax,ODEJointMax)

Type ODEGeom
	Field body
	Field geom
	Field mesh
End Type

; ###################################################################################################

; ### Setup ODE

dInitODE()

Global World=dWorldCreate()
Global Space=dHashSpaceCreate(0)
Global ContactGroup=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.4)
dContactSetMu(1)

; ### Create Light

Global Light=CreateLight()
RotateEntity Light,50,-50,0
LightColor Light,255,255,255
AmbientLight 130,130,130

; ### Create Camera

Global Camera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,9,-4
RotateEntity Camera,30,0,0

; ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture()

CreateMirror()

; ###################################################################################################

; ### Create RagDolls

For count=1 To 3
	CreateRagDoll(count)
Next

; ###################################################################################################

While Not KeyHit(1)

	UpdateControls()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime#=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+dGetVersion()
	Text 0,15,"Physics Time:"+PhysicsTime
	Text 300,0,"F1 - Drop RagDoll 1"
	Text 300,15,"F2 - Drop RagDoll 2"
	Text 300,30,"F3 - Drop RagDoll 3"
	Text 300,50,"Space - Throw RagDolls"
	Text 300,65,"Arrow Keys - Move Camera"
	Text 580,0,"F4 - Attach RagDoll 1"
	Text 580,15,"F5 - Attach RagDoll 2"
	Text 580,30,"F6 - Attach RagDoll 3"
	Text 580,50,"L Mouse Button - Push Body"
	Text 580,65,"R Mouse Button - Pull Body"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

; ###################################################################################################

Function CreateRagDoll(rd)

xp#=-8+(rd*4)
yp#=7
zp#=4

; ### RagDoll Dimensions

HeadRad#=0.4
BodyUX#=1.4
BodyUY#=1.5
BodyUZ#=0.5
BodyLX#=1.4
BodyLY#=0.75
BodyLZ#=0.5
ArmURad#=0.18
ArmULen#=1.1
ArmLRad#=0.18
ArmLLen#=1.1
LegURad#=0.25
LegULen#=1.3
LegLRad#=0.25
LegLLen#=1.3
FeetX#=0.5
FeetY#=0.25
FeetZ#=1.0
HandX#=0.1
HandY#=0.4
HandZ#=0.28

; ### Create Head

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,0)=ode\body
dBodySetPosition(ode\body,xp,yp,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateSphere(Space,HeadRad)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateSphere()
ScaleMesh ode\mesh,HeadRad,HeadRad,HeadRad
EntityColor ode\mesh,255,255,255
EntityPickMode ode\mesh,2

; ### Create Body (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,1)=ode\body
dBodySetPosition(ode\body,xp,yp-1.25,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,BodyUX,BodyUY,BodyUZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,BodyUX*0.5,BodyUY*0.5,BodyUZ*0.5
EntityColor ode\mesh,0,200,200
EntityPickMode ode\mesh,2

; ### Create Body (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,2)=ode\body
dBodySetPosition(ode\body,xp,yp-2.5,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,BodyLX,BodyLY,BodyLZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,BodyLX*0.5,BodyLY*0.5,BodyLZ*0.5
EntityColor ode\mesh,0,100,100
EntityPickMode ode\mesh,2

; ### Create Left Arm (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,3)=ode\body
dBodySetPosition(ode\body,xp+0.9,yp-1.25,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmURad,ArmULen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,ArmURad,ArmULen*0.5,ArmURad
EntityColor ode\mesh,255,0,0
EntityPickMode ode\mesh,2

; ### Create Left Arm (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,4)=ode\body
dBodySetPosition(ode\body,xp+0.9,yp-2.35,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmLRad,ArmLLen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,ArmLRad,ArmLLen*0.5,ArmLRad
EntityColor ode\mesh,125,0,0
EntityPickMode ode\mesh,2

; ### Create Left Hand

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,5)=ode\body
dBodySetPosition(ode\body,xp+0.9,yp-3.05,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,HandX,HandY,HandZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,HandX*0.5,HandY*0.5,HandZ*0.5
EntityColor ode\mesh,255,255,255
EntityPickMode ode\mesh,2

; ### Create Right Arm (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,6)=ode\body
dBodySetPosition(ode\body,xp-0.9,yp-1.25,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmURad,ArmULen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,ArmURad,ArmULen*0.5,ArmURad
EntityColor ode\mesh,0,255,0
EntityPickMode ode\mesh,2

; ### Create Right Arm (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,7)=ode\body
dBodySetPosition(ode\body,xp-0.9,yp-2.35,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,ArmLRad,ArmLLen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,ArmLRad,ArmLLen*0.5,ArmLRad
EntityColor ode\mesh,0,125,0
EntityPickMode ode\mesh,2

; ### Create Right Hand

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,8)=ode\body
dBodySetPosition(ode\body,xp-0.9,yp-3.05,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,HandX,HandY,HandZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,HandX*0.5,HandY*0.5,HandZ*0.5
EntityColor ode\mesh,255,255,255
EntityPickMode ode\mesh,2

; ### Create Left Leg (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,9)=ode\body
dBodySetPosition(ode\body,xp+0.5,yp-3.6,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegURad,LegULen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,LegURad,LegULen*0.5,LegURad
EntityColor ode\mesh,255,0,0
EntityPickMode ode\mesh,2

; ### Create Left Leg (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,10)=ode\body
dBodySetPosition(ode\body,xp+0.5,yp-4.9,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegLRad,LegLLen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,LegLRad,LegLLen*0.5,LegLRad
EntityColor ode\mesh,125,0,0
EntityPickMode ode\mesh,2

; ### Create Left Foot

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,11)=ode\body
dBodySetPosition(ode\body,xp+0.5,yp-5.7,zp-0.25)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,FeetX,FeetY,FeetZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,FeetX*0.5,FeetY*0.5,FeetZ*0.5
EntityColor ode\mesh,255,255,255
EntityPickMode ode\mesh,2

; ### Create Right Leg (Upper)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,12)=ode\body
dBodySetPosition(ode\body,xp-0.5,yp-3.6,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegURad,LegULen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,LegURad,LegULen*0.5,LegURad
EntityColor ode\mesh,0,255,0
EntityPickMode ode\mesh,2

; ### Create Right Leg (Lower)

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,13)=ode\body
dBodySetPosition(ode\body,xp-0.5,yp-4.9,zp)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateCylinder(Space,LegLRad,LegLLen)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCylinder()
ScaleMesh ode\mesh,LegLRad,LegLLen*0.5,LegLRad
EntityColor ode\mesh,0,125,0
EntityPickMode ode\mesh,2

; ### Create Right Foot

ode.ODEGeom=New ODEGeom
ode\body=dBodyCreate(World)
RDBody(rd,14)=ode\body
dBodySetPosition(ode\body,xp-0.5,yp-5.7,zp-0.25)
dBodySetRotation(ode\body,0,0,0)
dBodySetAutoDisableFlag(ode\body,1)
ode\geom=dCreateBox(Space,FeetX,FeetY,FeetZ)
dGeomSetBody(ode\geom,ode\body)
ode\mesh=CreateCube()
ScaleMesh ode\mesh,FeetX*0.5,FeetY*0.5,FeetZ*0.5
EntityColor ode\mesh,255,255,255
EntityPickMode ode\mesh,2

; ### Static Joint To Head

RDJoint(rd,0)=dJointCreateBall(World,0)
dJointAttach(RDJoint(rd,0),0,RDBody(rd,0))
dJointSetBallAnchor(RDJoint(rd,0),xp,yp,zp)

; ### Head To Body (Upper)

RDJoint(rd,1)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,1),RDBody(rd,0),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,1),1,0,1)
dJointSetHingeAnchor(RDJoint(rd,1),xp,yp-0.44,zp)
dJointSetHingeParam(RDJoint(rd,1),dParamLoStop,-0.7)
dJointSetHingeParam(RDJoint(rd,1),dParamHiStop,0.7)

; ### Body (Upper) To Body (Lower)

RDJoint(rd,2)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,2),RDBody(rd,1),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,2),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,2),xp,yp-2.05,zp)
dJointSetHingeParam(RDJoint(rd,2),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,2),dParamHiStop,0.25)

; ### Left Arm (Upper) To Body (Upper)

RDJoint(rd,3)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,3),RDBody(rd,3),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,3),1,-1,1)
dJointSetHingeAnchor(RDJoint(rd,3),xp+0.9,yp-0.7,zp)
dJointSetHingeParam(RDJoint(rd,3),dParamLoStop,-1.5)
dJointSetHingeParam(RDJoint(rd,3),dParamHiStop,1.5)

; ### Left Arm (Lower) To Left Arm (Upper)

RDJoint(rd,4)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,4),RDBody(rd,4),RDBody(rd,3))
dJointSetHingeAxis(RDJoint(rd,4),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,4),xp+0.9,yp-1.8,zp)
dJointSetHingeParam(RDJoint(rd,4),dParamLoStop,0.0)
dJointSetHingeParam(RDJoint(rd,4),dParamHiStop,2.0)

; ### Left Hand To Left Arm (Lower)

RDJoint(rd,5)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,5),RDBody(rd,5),RDBody(rd,4))
dJointSetHingeAxis(RDJoint(rd,5),0,0,1)
dJointSetHingeAnchor(RDJoint(rd,5),xp+0.9,yp-2.9,zp)
dJointSetHingeParam(RDJoint(rd,5),dParamLoStop,-0.3)
dJointSetHingeParam(RDJoint(rd,5),dParamHiStop,0.3)

; ### Right Arm (Upper) To Body (Upper)

RDJoint(rd,6)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,6),RDBody(rd,6),RDBody(rd,1))
dJointSetHingeAxis(RDJoint(rd,6),1,1,1)
dJointSetHingeAnchor(RDJoint(rd,6),xp-0.9,yp-0.7,zp)
dJointSetHingeParam(RDJoint(rd,6),dParamLoStop,-1.5)
dJointSetHingeParam(RDJoint(rd,6),dParamHiStop,1.5)

; ### Right Arm (Lower) To Right Arm (Upper)

RDJoint(rd,7)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,7),RDBody(rd,7),RDBody(rd,6))
dJointSetHingeAxis(RDJoint(rd,7),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,7),xp-0.9,yp-1.8,zp)
dJointSetHingeParam(RDJoint(rd,7),dParamLoStop,0.0)
dJointSetHingeParam(RDJoint(rd,7),dParamHiStop,2.0)

; ### Right Hand To Right Arm (Lower)

RDJoint(rd,8)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,8),RDBody(rd,8),RDBody(rd,7))
dJointSetHingeAxis(RDJoint(rd,8),0,0,1)
dJointSetHingeAnchor(RDJoint(rd,8),xp-0.9,yp-2.9,zp)
dJointSetHingeParam(RDJoint(rd,8),dParamLoStop,-0.3)
dJointSetHingeParam(RDJoint(rd,8),dParamHiStop,0.3)

; ### Left Leg (Upper) To Body (Lower)

RDJoint(rd,9)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,9),RDBody(rd,9),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,9),1,0,-1)
dJointSetHingeAnchor(RDJoint(rd,9),xp+0.5,yp-2.9,zp)
dJointSetHingeParam(RDJoint(rd,9),dParamLoStop,-1.0)
dJointSetHingeParam(RDJoint(rd,9),dParamHiStop,1.0)

; ### Left Leg (Lower) To Left Leg (Upper)

RDJoint(rd,10)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,10),RDBody(rd,10),RDBody(rd,9))
dJointSetHingeAxis(RDJoint(rd,10),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,10),xp+0.5,yp-4.25,zp)
dJointSetHingeParam(RDJoint(rd,10),dParamLoStop,-2.0)
dJointSetHingeParam(RDJoint(rd,10),dParamHiStop,0.0)

; ### Left Foot To Left Leg (Lower)

RDJoint(rd,11)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,11),RDBody(rd,11),RDBody(rd,10))
dJointSetHingeAxis(RDJoint(rd,11),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,11),xp+0.5,yp-5.5,zp)
dJointSetHingeParam(RDJoint(rd,11),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,11),dParamHiStop,0.25)

; ### Right Leg (Upper) To Body (Lower)

RDJoint(rd,12)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,12),RDBody(rd,12),RDBody(rd,2))
dJointSetHingeAxis(RDJoint(rd,12),1,0,1)
dJointSetHingeAnchor(RDJoint(rd,12),xp-0.5,yp-2.9,zp)
dJointSetHingeParam(RDJoint(rd,12),dParamLoStop,-1.0)
dJointSetHingeParam(RDJoint(rd,12),dParamHiStop,1.0)

; ### Right Leg (Lower) To Right Leg (Upper)

RDJoint(rd,13)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,13),RDBody(rd,13),RDBody(rd,12))
dJointSetHingeAxis(RDJoint(rd,13),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,13),xp-0.5,yp-4.25,zp)
dJointSetHingeParam(RDJoint(rd,13),dParamLoStop,-2.0)
dJointSetHingeParam(RDJoint(rd,13),dParamHiStop,0.0)

; ### Right Foot To Right Leg (Lower)

RDJoint(rd,14)=dJointCreateHinge(World,0)
dJointAttach(RDJoint(rd,14),RDBody(rd,14),RDBody(rd,13))
dJointSetHingeAxis(RDJoint(rd,14),1,0,0)
dJointSetHingeAnchor(RDJoint(rd,14),xp-0.5,yp-5.5,zp)
dJointSetHingeParam(RDJoint(rd,14),dParamLoStop,-0.25)
dJointSetHingeParam(RDJoint(rd,14),dParamHiStop,0.25)

End Function

; ###################################################################################################

Function UpdateControls()

If KeyDown(59)=1 Then dJointAttach(RDJoint(1,0),0,0)
If KeyDown(60)=1 Then dJointAttach(RDJoint(2,0),0,0)
If KeyDown(61)=1 Then dJointAttach(RDJoint(3,0),0,0)

If KeyDown(62)=1 Then dJointAttach(RDJoint(1,0),0,RDBody(1,0))
If KeyDown(63)=1 Then dJointAttach(RDJoint(2,0),0,RDBody(2,0))
If KeyDown(64)=1 Then dJointAttach(RDJoint(3,0),0,RDBody(3,0))

If KeyDown(57)=1
	dBodyAddForce RDBody(1,0),0,20,0
	dBodyAddForce RDBody(2,0),0,20,0
	dBodyAddForce RDBody(3,0),0,20,0
	End If

If KeyDown(200)=1 Then PositionEntity Camera,EntityX(Camera),EntityY(Camera),EntityZ(Camera)+0.3
If KeyDown(208)=1 Then PositionEntity Camera,EntityX(Camera),EntityY(Camera),EntityZ(Camera)-0.3
If KeyDown(203)=1 Then PositionEntity Camera,EntityX(Camera)-0.3,EntityY(Camera),EntityZ(Camera)
If KeyDown(205)=1 Then PositionEntity Camera,EntityX(Camera)+0.3,EntityY(Camera),EntityZ(Camera)

If MouseDown(1)=1
	CameraPick(Camera,MouseX(),MouseY())
	If PickedEntity()
		For ode.ODEGeom=Each ODEGeom
			If ode\mesh=PickedEntity()
				dBodyEnable(ode\body)
				dBodyAddForce ode\body,-20*PickedNX#(),-20*PickedNY#(),-20*PickedNZ#()
				End If
		Next
		End If
	End If

If MouseDown(2)=1
	CameraPick(Camera,MouseX(),MouseY())
	If PickedEntity()
		For ode.ODEGeom=Each ODEGeom
			If ode\mesh=PickedEntity()
				dBodyEnable(ode\body)
				dBodyAddForce ode\body,20*PickedNX#(),20*PickedNY#(),20*PickedNZ#()
				End If
		Next
		End If
	End If

End Function

; ###################################################################################################

Function UpdateGeoms()

For ode.ODEGeom=Each ODEGeom
	PositionEntity ode\mesh,dGeomGetPositionX(ode\geom),dGeomGetPositionY(ode\geom),dGeomGetPositionZ(ode\geom)
	RotateEntity ode\mesh,dGeomGetPitch(ode\geom),dGeomGetYaw(ode\geom),dGeomGetRoll(ode\geom)
Next

End Function

; ###################################################################################################
